%% Script to perform a simulation of detection of MITM attacks on intervehicle communication in a collaborative vehicle platoon (CVP)
%Results obtained using this code are presented in the PhD dissertation of
% Twan Keijzer: Advances in Safety and Security of Cyber-Physical Systems - Sliding Mode Observers, Coalitional Control and Homomorphic Encryption
clear all; close all; clc
%% Settings
RunSim = 1; %Set to 1 if you want to run the sim; or 0 if you want to load data
SaveData = 0; %Set to 1 if you want to save the data from the simulation
%!SavePlots is set just before plots are generated (line 183)!
EOIDetect = 0; %Set to 1 if you want to detect using the EOI, set to 0 if you want to detect using the 'direct error based method'
UncSweep = 0; %Set to 1 if you want to compare detection for different uncertainty magnitudes
AttackSweep = 0;%Set to 1 if you want to compare detection for different attack magnitudes

SensorVariances = [0:0.01:0.12 0.121:0.001:0.145 0.15:0.005:0.53]; %range of sensor variances used if UncSweep=1
AttackMags = [0.85:0.1:1.25 1.26:0.01:1.3 1.35:0.05:2.4 2.5:0.5:5 6:10];%range of attack magnitudes used if AttackSweep=1
ParameterSet={{diag([2.3500 3.300 2.160 3.6000]),0.1*eye(4)},...
              {diag([10.3500 11.300 10.160 11.6000]),2*eye(4)}...
              };%different parameter sets for which the simulation is run (only if we do sweep)
          
if RunSim && ~(UncSweep || AttackSweep)
    set_CACC_nocontrol%do main initialization now if doing a simulation and not doing a sweep, otherwise, do it later.
end

%generalize sweep variable
if UncSweep
    Sweeps = SensorVariances;
end
if AttackSweep
    Sweeps = AttackMags;
end
%% Saving and Loading
%where to save data and plots (if SaveData=1 and/or SavePlots=1)
SavePathData = 'Outputs/Data/';
SavePathPlots = 'Outputs/Figures/';
SaveName = 'SimScenario';

%where to load data from (if RunSim=0)
LoadPath = 'Outputs/Data/sim_Thesis/SimScenario.mat';

%basic plot formatting settings
Fontsize = 16;
Paperposition = [0 0 6 3];
%% Run Simulation
if RunSim
    if UncSweep || AttackSweep
        i=1;
        for sweep = Sweeps%for all uncertainty or attack magnitudes
            if UncSweep
                noise = sweep;
            end
            if AttackSweep
                attack_mag = sweep;
            end
            i2=1;
            for parameters = ParameterSet %for all parameter sets
            if UncSweep
                sensor.var = repmat(noise,4,1).^2;
            end
            M = parameters{1}{1};
            K = parameters{1}{2};
            set_CACC_nocontrol%do initialisation if we do a sweep.
            sim('CACC_nocontrol.slx')%do simulation
            %% Gather Data
            Data = simout.signals.values;
            j=1;
            %parse data
            size_ = size(Data);
            if length(size_)>2
                Data = squeeze(Data);
                size_ = size(Data);
                if length(size_)>2
                    error('incorrect dimensions')
                end
            end
            if size_(2)>size_(1)
                Data = Data';
            end
            %name variables
            u_l = Data(:,j);j=j+1;
            u_f = Data(:,j);j=j+1;
            du = Data(:,j);j=j+1;
            du_est = Data(:,j);j=j+1;
            threshold = Data(:,j:j+3);j=j+4;
            %output order depends on used detection method
            if ~EOIDetect
                Data(:,j) = simout_err.signals.values(:,1);
            end
            detect_flag = Data(:,j);j=j+1;
            nu_fil = Data(:,j:j+3);j=j+4;
            min_mag = Data(:,j:j+1);j=j+2;
            threshold_var_up = Data(:,j:j+3);j=j+4;
            threshold_var_down = Data(:,j:j+3);j=j+4;
            min_time = Data(:,j);j=j+1;
            p_l = Data(:,j);j=j+1;
            v_l = Data(:,j);j=j+1;
            a_l = Data(:,j);j=j+1;
            p_f = Data(:,j);j=j+1;
            v_f = Data(:,j);j=j+1;
            a_f = Data(:,j);j=j+1;
            bound_up = simout_err.signals.values(:,3);
            bound_down = simout_err.signals.values(:,7);
            time = tout;
            if EOIDetect
                if isempty(min(time(nu_fil(:,2)<-threshold(:,2))))
                    t_ds(i2,i) = Inf;
                else
                    t_ds(i2,i) = min(time(nu_fil(:,2)<-threshold(:,2)));
                end
            else
                if isempty(min(time(find(detect_flag==1,1))))
                    t_ds(i2,i) = Inf;
                else
                    t_ds(i2,i) = min(time(find(detect_flag==1,1)));
                end
            end
            %calculate when a crash occurs.
            t_crashes = time(abs(p_l-p_f)<L_l);
            if isempty(t_crashes)
                t_crash(i) = Inf;
            else
                t_crash(i) = t_crashes(1);
            end
            i2=i2+1;
            end
            i=i+1;
            Simulink.sdi.clear%clear simulink data inspector
        end
    else
        sim('CACC_nocontrol.slx')%if we do no sweep, just do the simulation once
        %% Gather Data
        Data = simout.signals.values;
        j=1;
        %parse data
        size_ = size(Data);
        if length(size_)>2
            Data = squeeze(Data);
            size_ = size(Data);
            if length(size_)>2
                error('incorrect dimensions')
            end
        end
        if size_(2)>size_(1)
            Data = Data';
        end
        %name variables
        u_l = Data(:,j);j=j+1;
        u_f = Data(:,j);j=j+1;
        du = Data(:,j);j=j+1;
        du_est = Data(:,j);j=j+1;
        threshold = Data(:,j:j+3);j=j+4;
        %output order depends on used detection method
        if ~EOIDetect
                Data(:,j) = simout_err.signals.values(:,1);
        end
        detect_flag = Data(:,j);j=j+1;
        nu_fil = Data(:,j:j+3);j=j+4;
        min_mag = Data(:,j:j+1);j=j+2;
        threshold_var_up = Data(:,j:j+3);j=j+4;
        threshold_var_down = Data(:,j:j+3);j=j+4;
        min_time = Data(:,j);j=j+1;
        p_l = Data(:,j);j=j+1;
        v_l = Data(:,j);j=j+1;
        a_l = Data(:,j);j=j+1;
        p_f = Data(:,j);j=j+1;
        v_f = Data(:,j);j=j+1;
        a_f = Data(:,j);j=j+1;
        bound_up = simout_err.signals.values(:,3);
        bound_down = simout_err.signals.values(:,7);
        time = tout;
    end
    %clear all variables that are no longer relevant
clear A_22s A_c A_e A_f A_l attack_mag B_c B_f B_l C_c const D_c D_e Data e2dot_min e_10 E_e h input_mag input_time j kd kp logsout maxacc_l n obs p r sensor simout simout_err size_ stepsize T_1 T_A T_B T_C t_max T_obs tau_f tau_l U_max U_max2
else
    load(LoadPath)%load data if we dont do a new simulation
end

%% Save Data
if SaveData
    if ~exist(SavePathData,'dir')
        mkdir(SavePathData)
    end
    save([SavePathData SaveName]);
end

SavePlots = 1;
%% Make & Save Plots
if SavePlots
    if ~exist(SavePathPlots,'dir')
        mkdir(SavePathPlots)
    end
end
%scenario plot
figure()
hold on
grid on
plot(time,u_l,'k-.','LineWidth',2)
plot(time,du,'r','LineWidth',2)
lgd = legend('$u_{(1)}$','$f$','Interpreter','latex','Location','north');
lgd.FontSize = Fontsize;
xlbl = xlabel('$t~[s]$','Interpreter','latex');
xlbl.FontSize = Fontsize;
ylbl = ylabel('$u~[m/s^2]$','Interpreter','latex');
ylbl.FontSize = Fontsize;
set(gca,'FontSize',Fontsize)
set(gcf,'PaperPosition',Paperposition);
ylim([-1.5,3.6]);
pbaspect([4,1,1])
if SavePlots
    print([SavePathPlots SaveName '_Scenario'],'-dpdf')
end
if SavePlots
    savefig([SavePathPlots SaveName '_Scenario'])
end

if EOIDetect
%EOI attack estimate plot
figure()
hold on
grid on
plot(time,du,'r','LineWidth',2)
plot(time,du_est,'b')
plot(time,min_mag(:,1),'Color','k','LineStyle','-.')
plot(time,min_mag(:,2),'Color','k','LineStyle','--')
detect = time(nu_fil(:,2)<-threshold(:,2));
if ~isempty(detect)
plot(repmat(detect(1),2,1),[-1,10],'Color',[0.14 0.55 0.14])
end
ylim([-0.5,3.6]);
lgd = legend('$\Delta u_{i-1}$','$\hat{\Delta} u_{i-1}$','$\Delta u_\mathrm{detect,nonoise}$','$\Delta u_\mathrm{detect,noise}$','$t_{\mathrm{detect}}$','Interpreter','latex');
lgd.Location = 'southeast';
lgd.FontSize = Fontsize;
xlbl = xlabel('$t~[s]$','Interpreter','latex');
xlbl.FontSize = Fontsize;
ylbl = ylabel('$u~[m/s^2]$','Interpreter','latex');
ylbl.FontSize = Fontsize;
set(gca,'FontSize',Fontsize)
set(gcf,'PaperPosition',Paperposition+[0 0 0 1]);
pbaspect([4,1,1])
if SavePlots
    print([SavePathPlots SaveName '_AttackEst'],'-dpdf')
end
if SavePlots
    savefig([SavePathPlots SaveName '_AttackEst'])
end
%EOI based attack detection plot
figure()
hold on
grid on
plot(time,nu_fil(:,2),'b')
plot(time,threshold_var_down(:,2),'Color',[0.4940 0.1840 0.5560],'LineStyle','-','LineWidth',2)
plot(time,-threshold(:,2),'Color',[0.4940 0.1840 0.5560],'LineStyle','--','LineWidth',2)
    t_d = time(nu_fil(:,2)<-threshold(:,2));
    if ~isempty(t_d)
    diftime = diff(t_d);
    I = find(diftime>0.0011);
    if isempty(I)
        I = length(t_d)-1;
    end
    for i=1:length(I)+1
        if i==1
            hdl = fill([t_d([1,I(i)])' fliplr(t_d([1,I(i)])')],[ones(1,2)*1000, -ones(1,2)*1000],'r','LineStyle','none');
        elseif i==length(I)+1
            hdl = fill([t_d([I(i-1)+1,end])' fliplr(t_d([I(i-1)+1,end])')],[ones(1,2)*1000, -ones(1,2)*1000],'r','LineStyle','none');
        else
            hdl = fill([t_d([I(i-1)+1,I(i)])' fliplr(t_d([I(i-1)+1,I(i)])')],[ones(1,2)*1000, -ones(1,2)*1000],'r','LineStyle','none');
        end
        set(hdl,'facealpha',.25)
    end
    end
set(gca,'Ydir','reverse')
ylim([-3.6,0.5]);
lgd = legend('$\nu_{\mathrm{eq},(2)}$','$\underline{\nu}_{\mathrm{eq},(2)}$','$\underline{\nu}_{\mathrm{eq},(2)}^{\mbox{const}}$','Detection','Interpreter','latex','Orientation','horizontal');%,'$t_\mathrm{detect}$','$t_\mathrm{detect,max}$';%'$\underline{\nu}_{\mathrm{fil}(2)}^*$','$t_\mathrm{detect}$','$t_\mathrm{detect,max}$','Interpreter','latex');%legend('$\nu_\mathrm{fil}(1)$','$\nu_\mathrm{fil}(2)$','$\nu_\mathrm{fil}(3)$','$\nu_\mathrm{fil}(4)$','Detection','Guar. Detect. Time','Interpreter','latex');
lgd.Location = 'northoutside';
lgd.FontSize = Fontsize;
xlbl = xlabel('$t~[s]$','Interpreter','latex');
xlbl.FontSize = Fontsize;
ylbl = ylabel('$\nu_{\mathrm{eq},(2)}$','Interpreter','latex');
ylbl.FontSize = Fontsize;
set(gca,'FontSize',Fontsize)
set(gcf,'PaperPosition',Paperposition+[0 0 0 1]);
pbaspect([4,1,1])
if SavePlots
    print([SavePathPlots SaveName '_Detection'],'-dpdf')
end
if SavePlots
    savefig([SavePathPlots SaveName '_Detection'])
end
else
%direct error based attack detection plot    
figure()
hold on
grid on
plot(time,bound_up,'Color',[0.6940 0.3 0.7560],'LineStyle','-','LineWidth',2)
plot(time,bound_down,'Color',[0.2940 0.0840 0.3560],'LineStyle','-','LineWidth',2)
    t_d = time(detect_flag==1);
    if ~isempty(t_d)
    diftime = diff(t_d);
    I = find(diftime>0.0011);
    if isempty(I)
        I = length(t_d)-1;
    end
    for i=1:length(I)+1
        if i==1
            hdl = fill([t_d([1,I(i)])' fliplr(t_d([1,I(i)])')],[ones(1,2)*1000, -ones(1,2)*1000],'r','LineStyle','none');
        elseif i==length(I)+1
            hdl = fill([t_d([I(i-1)+1,end])' fliplr(t_d([I(i-1)+1,end])')],[ones(1,2)*1000, -ones(1,2)*1000],'r','LineStyle','none');
        else
            hdl = fill([t_d([I(i-1)+1,I(i)])' fliplr(t_d([I(i-1)+1,I(i)])')],[ones(1,2)*1000, -ones(1,2)*1000],'r','LineStyle','none');
        end
        set(hdl,'facealpha',.25)
    end
    end
ylim([-2,0.5]);
lgd = legend('$\bar{\epsilon}_{2,(2)}$','$\underline{\epsilon}_{2,(2)}$','Detection','Interpreter','latex','Orientation','horizontal');%,'$t_\mathrm{detect}$','$t_\mathrm{detect,max}$';%'$\underline{\nu}_{\mathrm{fil}(2)}^*$','$t_\mathrm{detect}$','$t_\mathrm{detect,max}$','Interpreter','latex');%legend('$\nu_\mathrm{fil}(1)$','$\nu_\mathrm{fil}(2)$','$\nu_\mathrm{fil}(3)$','$\nu_\mathrm{fil}(4)$','Detection','Guar. Detect. Time','Interpreter','latex');
lgd.Location = 'northoutside';
lgd.FontSize = Fontsize;
xlbl = xlabel('$t~[s]$','Interpreter','latex');
xlbl.FontSize = Fontsize;
ylbl = ylabel('$-~[m/s]$','Interpreter','latex');
ylbl.FontSize = Fontsize;
set(gca,'FontSize',Fontsize)
set(gcf,'PaperPosition',Paperposition+[0 0 0 1]);
pbaspect([4,1,1])
if SavePlots
    print([SavePathPlots SaveName '_Detection'],'-dpdf')
end
if SavePlots
    savefig([SavePathPlots SaveName '_Detection'])
end
%direct error based detection zoomed plot
figure()
hold on
grid on
plot(time,bound_up,'Color',[0.6940 0.3 0.7560],'LineStyle','-','LineWidth',2)
plot(time,bound_down,'Color',[0.2940 0.0840 0.3560],'LineStyle','-','LineWidth',2)
for i=1:length(I)+1
    if i==1
        hdl = fill([t_d([1,I(i)])' fliplr(t_d([1,I(i)])')],[ones(1,2)*1000, -ones(1,2)*1000],'r','LineStyle','none');
    elseif i==length(I)+1
        hdl = fill([t_d([I(i-1)+1,end])' fliplr(t_d([I(i-1)+1,end])')],[ones(1,2)*1000, -ones(1,2)*1000],'r','LineStyle','none');
    else
        hdl = fill([t_d([I(i-1)+1,I(i)])' fliplr(t_d([I(i-1)+1,I(i)])')],[ones(1,2)*1000, -ones(1,2)*1000],'r','LineStyle','none');
    end
    set(hdl,'facealpha',.25)
end
xlim([47,50]);
ylim([-0.5,0.5]);
lgd = legend('$\bar{\epsilon}_{2,(2)}$','$\underline{\epsilon}_{2,(2)}$','Detection','Interpreter','latex','Orientation','horizontal');%,'$t_\mathrm{detect}$','$t_\mathrm{detect,max}$';%'$\underline{\nu}_{\mathrm{fil}(2)}^*$','$t_\mathrm{detect}$','$t_\mathrm{detect,max}$','Interpreter','latex');%legend('$\nu_\mathrm{fil}(1)$','$\nu_\mathrm{fil}(2)$','$\nu_\mathrm{fil}(3)$','$\nu_\mathrm{fil}(4)$','Detection','Guar. Detect. Time','Interpreter','latex');
lgd.Location = 'northoutside';
lgd.FontSize = Fontsize;
xlbl = xlabel('$t~[s]$','Interpreter','latex');
xlbl.FontSize = Fontsize;
ylbl = ylabel('$-~[m/s]$','Interpreter','latex');
ylbl.FontSize = Fontsize;
set(gca,'FontSize',Fontsize)
set(gcf,'PaperPosition',Paperposition+[0 0 0 1]);
pbaspect([4,1,1])
if SavePlots
    print([SavePathPlots SaveName '_Detection_zoom'],'-dpdf')
end
if SavePlots
    savefig([SavePathPlots SaveName '_Detection_zoom'])
end
end
if UncSweep
    %uncertainty sweep detection time plot
    figure()
    grid on
    hold on
    plot(3*sqrt(SensorVariances.^2),t_ds(1,:)-attack_time,'LineStyle','-','LineWidth',2);
    plot(3*sqrt(SensorVariances.^2),t_ds(2,:)-attack_time,'LineStyle','--','LineWidth',2);
    lgd = legend('Slow','Fast','Interpreter','latex','Orientation','horizontal');
    lgd.Location = 'northwest';
    lgd.FontSize = Fontsize;
    xlbl = xlabel('$\bar{\zeta}_2$','Interpreter','latex');
    xlbl.FontSize = Fontsize;
    ylbl = ylabel('Detection Time $[s]$','Interpreter','latex');
    ylbl.FontSize = Fontsize;
    set(gca,'FontSize',Fontsize)
    set(gcf,'PaperPosition',Paperposition+[0 0 0 1]);
    ylim([0,4]);
    xlim([0,1.6]);
    pbaspect([4,1,1])
    if SavePlots
        print([SavePathPlots SaveName '_UncSweep'],'-dpdf')
    end
    if SavePlots
        savefig([SavePathPlots SaveName '_UncSweep'])
    end
end
if AttackSweep
    %attack sweep detection time plot
    figure()
    grid on
    hold on
    plot(AttackMags,t_ds(1,:)-attack_time,'k','LineWidth',2);
    plot(AttackMags,t_crash-attack_time,'r--','LineWidth',2)
    for i=2:length(ParameterSet)
        plot(AttackMags,t_ds(i,:)-attack_time,'k','LineWidth',2);
    end
    lgd = legend('Detection','Crash','Interpreter','latex','Orientation','horizontal');
    lgd.Location = 'northeast';
    lgd.FontSize = Fontsize;
    xlbl = xlabel('Attack $[m/s^2]$','Interpreter','latex');
    xlbl.FontSize = Fontsize;
    ylbl = ylabel('Time $[s]$','Interpreter','latex');
    ylbl.FontSize = Fontsize;
    set(gca,'FontSize',Fontsize)
    set(gcf,'PaperPosition',Paperposition+[0 0 0 1]);
    ylim([0,25]);
    xlim([0,10]);
    pbaspect([4,1,1])
    if SavePlots
        print([SavePathPlots SaveName '_AttackSweep'],'-dpdf')
    end
    if SavePlots
        savefig([SavePathPlots SaveName '_AttackSweep'])
    end
end